@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2022-2023 Rupert Carmichael
+Copyright (c) 2022-2025 Rupert Carmichael
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <libgen.h>
 
 #include <jg/jg.h>
+#if JG_VERSION_NUMBER > 10000
 #include <jg/jg_neogeo.h>
+#else
+#include "jg_neogeo_geo.h"
+#endif
 
 #include "geo.h"
 #include "geo_lspc.h"
@@ -92,7 +96,7 @@ static jg_setting_t settings_geo[] = {
       "0 = Auto, 1 = Neo Geo Joysticks, 2 = Mahjong, "
       "3 = 4-Player (AS/JP MVS)",
       "Select the desired input device(s)",
-      0, 0, 3, JG_SETTING_RESTART
+      0, 0, 3, JG_SETTING_INPUT
     },
     { "system", "Emulated System",
       "0 = AES (Console), 1 = MVS (Arcade), 2 = Universe BIOS",
@@ -139,6 +143,12 @@ static jg_setting_t settings_geo[] = {
       "Hide N pixels of Overscan (Right)",
       2, 0, 4, 0
     },
+    { "adpcm_wrap", "ADPCM Accumulator Wrap",
+      "0 = Off, 1 = On",
+      "ADPCM Accumulator Wrap may be disabled to fix sound effects in buggy "
+      "games, for example Ganryu and Nightmare in the Dark. This is a hack!",
+      1, 0, 1, 0
+    },
 };
 
 enum {
@@ -152,18 +162,7 @@ enum {
     OVERSCAN_B,
     OVERSCAN_L,
     OVERSCAN_R,
-};
-
-static const char *defs_ngsys[4] = { "Coin1", "Coin2", "Service", "Test" };
-
-static const char *defs_mahjong[21] = {
-    "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N",
-    "Pon", "Chi", "Kan", "Reach", "Ron", "Select", "Start"
-};
-
-static const char *defs_vliner[11] = {
-    "Up", "Down", "Left", "Right", "Big", "Small", "D-Up", "Start",
-    "Operator", "ClearCredit", "HopperOut"
+    ADPCM_WRAP,
 };
 
 // Database flags for automatically setting inputs
@@ -391,6 +390,60 @@ static unsigned geo_input_poll_vliner(unsigned port) {
     return b;
 }
 
+static void geo_params_input(void) {
+    inputinfo[0] = jg_neogeo_inputinfo(0, JG_NEOGEO_JS);
+    inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_JS);
+    inputinfo[2] = jg_neogeo_inputinfo(2, JG_NEOGEO_UNCONNECTED);
+    inputinfo[3] = jg_neogeo_inputinfo(3, JG_NEOGEO_UNCONNECTED);
+    inputinfo[4] = jg_neogeo_inputinfo(4, JG_NEOGEO_SYS);
+
+    geo_input_set_callback(0, &geo_input_poll_js);
+    geo_input_set_callback(1, &geo_input_poll_js);
+    geo_input_sys_set_callback(0, &geo_input_poll_stat_a);
+    geo_input_sys_set_callback(1, &geo_input_poll_stat_b);
+    geo_input_sys_set_callback(2, &geo_input_poll_systype);
+    geo_input_sys_set_callback(3, &geo_input_poll_dipsw);
+
+    // Mahjong
+    if ((settings_geo[INPUT].val == 2) ||
+        ((dbflags & GEO_DB_MAHJONG) && (settings_geo[INPUT].val == 0))) {
+        dipswitches |= 0x04; // Set Mahjong mode
+        inputinfo[0] = jg_neogeo_inputinfo(0, JG_NEOGEO_MAHJONG);
+        inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_UNCONNECTED);
+        geo_input_set_callback(0, &geo_input_poll_mahjong);
+        geo_input_set_callback(1, &geo_input_poll_none);
+        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_mahjong);
+    }
+    else if (dbflags & GEO_DB_IRRMAZE) {
+        if (settings_geo[SYS].val != SYSTEM_UNI) {
+            jg_cb_log(JG_LOG_WRN, "The Irritating Maze must be played using "
+                "the Universe BIOS with the \"Enable Joystick Play\" option "
+                "set.\n");
+        }
+    }
+    else if (dbflags & GEO_DB_VLINER) {
+        inputinfo[0] = jg_neogeo_inputinfo(0, JG_NEOGEO_VLINER);
+        inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_UNCONNECTED);
+        geo_input_set_callback(0, &geo_input_poll_vliner);
+        geo_input_set_callback(1, &geo_input_poll_none);
+        geo_input_sys_set_callback(0, &geo_input_poll_stat_a_vliner);
+        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_vliner);
+        geo_input_sys_set_callback(4, &geo_input_poll_sys_vliner);
+    }
+    else if (settings_geo[INPUT].val == 3 &&
+        settings_geo[SYS].val == SYSTEM_MVS && (settings_geo[REGION].val ==
+        REGION_AS || settings_geo[REGION].val == REGION_JP)) {
+        // Set Four Player mode
+        dipswitches |= 0x02;
+
+        inputinfo[2] = jg_neogeo_inputinfo(2, JG_NEOGEO_JS);
+        inputinfo[3] = jg_neogeo_inputinfo(3, JG_NEOGEO_JS);
+        geo_input_set_callback(0, &geo_input_poll_js_ftc1b);
+        geo_input_set_callback(1, &geo_input_poll_js_ftc1b);
+        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_ftc1b);
+    }
+}
+
 static void geo_params_video(void) {
     unsigned w = LSPC_WIDTH - ((settings_geo[OVERSCAN_L].val << 2) +
         (settings_geo[OVERSCAN_R].val << 2));
@@ -429,6 +482,7 @@ int jg_init(void) {
     geo_log_set_callback(jg_cb_log);
     geo_set_region(settings_geo[REGION].val);
     geo_set_system(settings_geo[SYS].val);
+    geo_set_adpcm_wrap(settings_geo[ADPCM_WRAP].val);
     geo_mixer_set_rate(SAMPLERATE);
     geo_mixer_init();
     geo_params_video();
@@ -491,76 +545,7 @@ int jg_game_load(void) {
                 datatype[i], savename);
     }
 
-    inputinfo[0] = jg_neogeo_inputinfo(0, JG_NEOGEO_JS);
-    inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_JS);
-    inputinfo[2] = jg_neogeo_inputinfo(2, JG_NEOGEO_UNCONNECTED);
-    inputinfo[3] = jg_neogeo_inputinfo(3, JG_NEOGEO_UNCONNECTED);
-    //inputinfo[4] = jg_neogeo_inputinfo(2, JG_NEOGEO_SYS);
-    inputinfo[4] = (jg_inputinfo_t){
-        JG_INPUT_EXTERNAL, 4, "neogeosystem", "Neo Geo System",
-        defs_ngsys, 0, 4
-    };
-
-    geo_input_set_callback(0, &geo_input_poll_js);
-    geo_input_set_callback(1, &geo_input_poll_js);
-    geo_input_sys_set_callback(0, &geo_input_poll_stat_a);
-    geo_input_sys_set_callback(1, &geo_input_poll_stat_b);
-    geo_input_sys_set_callback(2, &geo_input_poll_systype);
-    geo_input_sys_set_callback(3, &geo_input_poll_dipsw);
-
-    // Mahjong
-    if ((settings_geo[INPUT].val == 2) ||
-        ((dbflags & GEO_DB_MAHJONG) && (settings_geo[INPUT].val == 0))) {
-        // Set Mahjong mode
-        dipswitches |= 0x04;
-
-        inputinfo[0] = (jg_inputinfo_t){
-            JG_INPUT_CONTROLLER, 0, "neogeomahjong",
-            "Neo Geo Mahjong Controller", defs_mahjong, 0, 21
-        };
-        inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_UNCONNECTED);
-        geo_input_set_callback(0, &geo_input_poll_mahjong);
-        geo_input_set_callback(1, &geo_input_poll_none);
-        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_mahjong);
-    }
-    else if (dbflags & GEO_DB_IRRMAZE) {
-        if (settings_geo[SYS].val != SYSTEM_UNI) {
-            jg_cb_log(JG_LOG_WRN, "The Irritating Maze must be played using "
-                "the Universe BIOS with the \"Enable Joystick Play\" option "
-                "set.\n");
-        }
-    }
-    else if (dbflags & GEO_DB_VLINER) {
-        inputinfo[0] = (jg_inputinfo_t){
-            JG_INPUT_CONTROLLER, 0, "neogeovliner",
-            "V-Liner", defs_vliner, 0, 11
-        };
-        inputinfo[1] = jg_neogeo_inputinfo(1, JG_NEOGEO_UNCONNECTED);
-        geo_input_set_callback(0, &geo_input_poll_vliner);
-        geo_input_set_callback(1, &geo_input_poll_none);
-        geo_input_sys_set_callback(0, &geo_input_poll_stat_a_vliner);
-        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_vliner);
-        geo_input_sys_set_callback(4, &geo_input_poll_sys_vliner);
-    }
-    else if (settings_geo[INPUT].val == 3 &&
-        settings_geo[SYS].val == SYSTEM_MVS && (settings_geo[REGION].val ==
-        REGION_AS || settings_geo[REGION].val == REGION_JP)) {
-        // Set Four Player mode
-        dipswitches |= 0x02;
-
-        geo_input_set_callback(0, &geo_input_poll_js_ftc1b);
-        geo_input_set_callback(1, &geo_input_poll_js_ftc1b);
-        geo_input_sys_set_callback(1, &geo_input_poll_stat_b_ftc1b);
-
-        inputinfo[2] = (jg_inputinfo_t){
-            JG_INPUT_CONTROLLER, 2, "neogeojs3", "Neo Geo Joystick",
-            defs_neogeojs, 0, NDEFS_NEOGEOJS
-        };
-        inputinfo[3] = (jg_inputinfo_t){
-            JG_INPUT_CONTROLLER, 3, "neogeojs4", "Neo Geo Joystick",
-            defs_neogeojs, 0, NDEFS_NEOGEOJS
-        };
-    }
+    geo_params_input();
 
     jg_cb_frametime(FRAMERATE);
 
@@ -629,7 +614,9 @@ void jg_cheat_set(const char *code) {
 }
 
 void jg_rehash(void) {
+    geo_params_input();
     geo_params_video();
+    geo_set_adpcm_wrap(settings_geo[ADPCM_WRAP].val);
 }
 
 void jg_data_push(uint32_t type, int port, const void *ptr, size_t size) {
